@@ -101,7 +101,7 @@ class TimingReport(builder.Builder):
         Returns:
             `str`: The formatted time, with 2+ digits for minutes and 2 digits for seconds.
         """
-        return '{:02}:{:02}'.format(*divmod(secs, 60))
+        return '{:02}:{:02}'.format(*divmod(int(secs), 60))
 
 
     def set_document_metadata(self, doc_structure, page_labels):
@@ -175,13 +175,17 @@ class TimingReport(builder.Builder):
 
         treemodel = Gtk.TreeStore(str, str, str, str)
 
+        npages = len(self.page_labels)
+        maxlen = len(str(npages))
+
         dfs_info = [(None, infos)]
         while dfs_info:
             first_it, first = dfs_info.pop()
             page = first['page']
             label = self.page_labels[page] if 0 <= page < len(self.page_labels) else 'None'
+            label += '\u2007' * (maxlen - len(str(page)))
 
-            last_col = '{} ({}/{})'.format(label, page, len(self.page_labels))
+            last_col = '{} ({}/{})'.format(label, page, npages)
             row = [first['title'], self.format_time(first['time']), self.format_time(first['duration']), last_col]
             it = treemodel.append(first_it, row)
 
@@ -226,6 +230,7 @@ class LayoutEditor(builder.Builder):
         'plain':      _('Plain layout, without note slides'),
         'note_pages': _('Layout for libreoffice notes on separate pages (with current slide preview in notes)'),
         'highlight':  _('Layout to draw on the current slide'),
+        'highlight_notes':  _('Layout to draw on the current slide with notes displayed'),
     }
 
     _model_columns = ['widget', 'has_resizeable', 'resizeable', 'has_orientation', 'orientation', 'next_slide_count',
