@@ -46,7 +46,8 @@ class Scribbler(builder.Builder):
     """
     #: Whether we are displaying the interface to scribble on screen and the overlays containing said scribbles
     scribbling_mode = False
-    #: `list` of scribbles to be drawn, as tuples of color :class:`~Gdk.RGBA`, width `int`, and a `list` of points.
+    #: `list` of scribbles to be drawn, as tuples of color :class:`~Gdk.RGBA`, width `int`, a `list` of points,
+    #: and a `list` of pressure values.
     scribble_list = []
     #: `list` of undone scribbles to possibly redo
     scribble_redo_list = []
@@ -187,9 +188,9 @@ class Scribbler(builder.Builder):
         self.eraser_surface = cairo.ImageSurface.create_from_png(str(util.get_icon_path('eraser.png')))
 
         # Load color and active pen preferences. Pen 0 is the eraser.
-        self.color_width = [(Gdk.RGBA(0, 0, 0, 0), 150)] + list(zip(
+        self.color_width = [(Gdk.RGBA(0, 0, 0, 0), config.getfloat('highlight', 'width_eraser'))] + list(zip(
             [self.parse_color(config.get('highlight', 'color_{}'.format(pen))) for pen in range(1, 10)],
-            [config.getint('highlight', 'width_{}'.format(pen)) for pen in range(1, 10)],
+            [config.getfloat('highlight', 'width_{}'.format(pen)) for pen in range(1, 10)],
         ))
 
         self.scribble_preset_buttons = [
@@ -294,15 +295,13 @@ class Scribbler(builder.Builder):
         if len(points) <= 2:
             return curves
 
-        c1 = points[1]
-        for c2, pt in zip(points[2:-1:2], points[3:-1:2]):
-            half_c2pt = (pt[0] - c2[0]) / 2, (pt[1] - c2[1]) / 2
-
-            curves.append((*c1, c2[0] + half_c2pt[0], c2[1] + half_c2pt[1], *pt))
-            c1 = (pt[0] + half_c2pt[0], pt[1] + half_c2pt[1])
-
-        if len(points) % 2 == 0:
-            curves.append((*c1, *points[-2], *points[-1]))
+        for (ax, ay), (bx, by), (cx, cy), (dx, dy) in zip(
+                [points[0], *points[:-2]], points[:-1], points[1:], [*points[2:], points[-1]]
+        ):
+            curves.append((bx, by,
+                           bx + (cx - ax) / 4, by + (cy - ay) / 4,
+                           cx + (bx - dx) / 4, cy + (by - dy) / 4,
+                           cx, cy))
 
         return curves
 
@@ -317,9 +316,12 @@ class Scribbler(builder.Builder):
         Returns:
             `bool`: whether the event was consumed
         """
-        pos = self.get_slide_point(widget, event)
+        pos = self.get_slide_point(widget, event) + ()
+
         if self.scribble_drawing:
-            self.scribble_list[-1][-1].append(pos)
+            self.scribble_list[-1][-2].append(pos)
+            pressure = event.get_axis(Gdk.AxisUse.PRESSURE)
+            self.scribble_list[-1][-1].append(1. if pressure is None else pressure)
             self.scribble_redo_list.clear()
 
             self.adjust_buttons()
@@ -374,13 +376,14 @@ class Scribbler(builder.Builder):
             return False
 
         if event.get_event_type() == Gdk.EventType.BUTTON_PRESS:
-            if any(mod & event.get_state() == mod for mod in self.toggle_erase_modifiers) and self.active_preset \
-                    and self.toggle_erase_source is None:
+            eraser_button = event.get_source_device().get_source() == Gdk.InputSource.ERASER
+            eraser_modifier = any(mod & event.get_state() == mod for mod in self.toggle_erase_modifiers)
+            if (eraser_button or eraser_modifier) and self.active_preset and self.toggle_erase_source is None:
                 self.previous_preset = self.active_preset
                 self.toggle_erase_source = 'modifier'
                 self.load_preset(target=0)
 
-            self.scribble_list.append((self.scribble_color, self.scribble_width, []))
+            self.scribble_list.append((self.scribble_color, self.scribble_width, [], []))
             self.scribble_drawing = True
 
             return self.track_scribble(widget, event)
@@ -404,7 +407,8 @@ class Scribbler(builder.Builder):
         window = self.c_da.get_window()
 
         if window is None:
-            return ValueError('Cannot initialize scribble cache without drawing area window')
+            logger.error('Cannot initialize scribble cache without drawing area window')
+            return
 
         scale = window.get_scale_factor()
         ww, wh = self.c_da.get_allocated_width() * scale, self.c_da.get_allocated_height() * scale
@@ -417,25 +421,23 @@ class Scribbler(builder.Builder):
                 try:
                     self.scribble_cache = window.create_similar_image_surface(cairo.Format.ARGB32, ww, wh, scale)
                 except ValueError:
-                    return
+                    logger.exception('Error creating highlight cache')
+                else:
+                    self.next_render = 0
             else:
                 logger.exception('Error creating highlight cache')
         except cairo.Error:
             logger.warning('Failed creating an ARGB32 surface sized {}x{} scale {} for highlight cache'
                            .format(ww, wh, scale), exc_info=True)
-            return
-        self.next_render = 0
+        else:
+            self.next_render = 0
 
 
     def prerender(self):
         """ Commit scribbles to cache so they are faster to draw on the slide
         """
         if self.scribble_cache is None:
-            try:
-                self.reset_scribble_cache()
-            except ValueError as e:
-                logger.info(e)
-                return
+            self.reset_scribble_cache()
 
         if self.scribble_cache is None:
             self.next_render = 0
@@ -450,15 +452,15 @@ class Scribbler(builder.Builder):
 
         draw = slice(self.next_render, -1 if self.scribble_drawing else None)
 
-        for color, width, points in self.scribble_list[draw]:
-            self.render_scribble(cairo_context, color, width * pen_scale_factor, [(x * ww, y * wh) for x, y in points])
-
+        for color, width, points, pressure in self.scribble_list[draw]:
+            self.render_scribble(cairo_context, color, width * pen_scale_factor, [(x * ww, y * wh) for x, y in points],
+                                 pressure)
         del cairo_context
 
-        self.next_render = len(self.scribble_list) + (draw.stop if draw.stop else 0)
+        self.next_render = draw.indices(len(self.scribble_list))[1]
 
 
-    def render_scribble(self, cairo_context, color, width, points):
+    def render_scribble(self, cairo_context, color, width, points, pressures):
         """ Draw a single scribble, i.e. a bezier curve, on the cairo context
 
         Args:
@@ -466,29 +468,38 @@ class Scribbler(builder.Builder):
             color (:class:`~Gdk.RGBA`): The color of the scribble
             width (`float`): The width of the curve
             points (`list`): The control points of the curve, scaled to the surface.
-
-        Returns:
-            :class:`~cairo.Path`: A copy of the path that was drawn
+            pressures (`list`): The relative line width at each point as `float` values in 0..1
         """
         if not points:
             return
 
-        # alpha == 0 -> Eraser mode
-        cairo_context.set_operator(cairo.OPERATOR_OVER if color.alpha else cairo.OPERATOR_CLEAR)
+        # Draw every stroke on a separate surface, then merge them all into the scribble cache
+        # Erasers do not have their own group as they are meant to interfere with strokes below
+        if color.alpha:
+            cairo_context.push_group()
+            cairo_context.set_operator(cairo.OPERATOR_SOURCE)
+        else:
+            # alpha == 0 -> Eraser mode
+            cairo_context.set_operator(cairo.OPERATOR_CLEAR)
         cairo_context.set_source_rgba(*color)
-        cairo_context.set_line_width(width)
 
-        cairo_context.move_to(*points[0])
-
-        for curve in self.points_to_curves(points):
-            cairo_context.curve_to(*curve)
-
-        path = cairo_context.copy_path()
+        curves = self.points_to_curves(points)
+        curve_widths = [(a + b) / 2 for a, b in zip(pressures[:-1], pressures[1:])]
+        for curve, relwidth in zip(curves, curve_widths):
+            cairo_context.move_to(*curve[:2])
+            cairo_context.set_line_width(width * relwidth)
+            cairo_context.curve_to(*curve[2:])
+            cairo_context.stroke()
 
+        # Draw from last uneven-indexed point to last point
+        cairo_context.move_to(*points[-2 if len(points) % 2 and len(points) > 1 else -1])
+        cairo_context.set_line_width(width * (curve_widths[-1] if curve_widths else pressures[-1]))
         cairo_context.line_to(*points[-1])
         cairo_context.stroke()
 
-        return path
+        if color.alpha:
+            cairo_context.pop_group_to_source()
+            cairo_context.paint()
 
 
     def draw_scribble(self, widget, cairo_context):
@@ -502,6 +513,8 @@ class Scribbler(builder.Builder):
         ww, wh = widget.get_allocated_width(), widget.get_allocated_height()
         cw, ch = self.scribble_cache.get_width(), self.scribble_cache.get_height()
 
+        cairo_context.push_group()
+
         cairo_context.save()
 
         cairo_context.scale(ww * scale / cw, wh * scale / ch)
@@ -510,13 +523,12 @@ class Scribbler(builder.Builder):
 
         cairo_context.restore()
 
-        cairo_context.push_group()
-
         pen_scale_factor = max(ww / 900, wh / 900)  # or sqrt of product
         if self.scribble_drawing:
             cairo_context.set_line_cap(cairo.LINE_CAP_ROUND)
-            color, width, points = self.scribble_list[-1]
-            self.render_scribble(cairo_context, color, width * pen_scale_factor, [(x * ww, y * wh) for x, y in points])
+            color, width, points, pressure = self.scribble_list[-1]
+            self.render_scribble(cairo_context, color, width * pen_scale_factor, [(x * ww, y * wh) for x, y in points],
+                                 pressure)
 
         cairo_context.pop_group_to_source()
         cairo_context.paint()
@@ -554,21 +566,20 @@ class Scribbler(builder.Builder):
             event (:class:`~Gdk.Event`):  the GTK event triggering this update.
             value (`int`): the width of the scribbles to be drawn
         """
-        self.scribble_width = max(5, min(90, int(value)))
+        self.scribble_width = max(1, min(100, 10 ** value if value < 1 else 10 + (value - 1) * 90))
         self.update_active_color_width()
 
 
     def update_active_color_width(self):
         """ Update modifications to the active scribble color and width, on the pen button and config object
         """
-        if not self.active_preset:
-            return
-
         self.color_width[self.active_preset] = self.scribble_color, self.scribble_width
-        self.scribble_preset_buttons[self.active_preset].queue_draw()
+        if self.active_preset:
+            self.scribble_preset_buttons[self.active_preset].queue_draw()
 
-        pen = self.active_preset
-        self.config.set('highlight', 'color_{}'.format(pen), self.scribble_color.to_string())
+        pen = self.active_preset if self.active_preset else 'eraser'
+        if self.active_preset:
+            self.config.set('highlight', 'color_{}'.format(pen), self.scribble_color.to_string())
         self.config.set('highlight', 'width_{}'.format(pen), str(self.scribble_width))
 
 
@@ -735,6 +746,9 @@ class Scribbler(builder.Builder):
         self.p_central.queue_draw()
         self.scribble_overlay.queue_draw()
 
+        # Get frequent events for smooth drawing
+        self.p_central.get_window().set_event_compression(False)
+
         self.scribbling_mode = True
         self.get_application().lookup_action('highlight').change_state(GLib.Variant.new_boolean(self.scribbling_mode))
         self.pen_action.set_enabled(self.scribbling_mode)
@@ -758,6 +772,9 @@ class Scribbler(builder.Builder):
         extras.Cursor.set_cursor(self.scribble_p_da, 'default')
         self.load_layout(None)
         self.scribble_off_render.add(self.scribble_overlay)
+        window = self.p_central.get_window()
+        if window:
+            window.set_event_compression(True)
 
         self.get_application().lookup_action('highlight').change_state(GLib.Variant.new_boolean(self.scribbling_mode))
         self.pen_action.set_enabled(self.scribbling_mode)
@@ -779,7 +796,7 @@ class Scribbler(builder.Builder):
         Returns:
             `bool`: whether the preset was loaded
         """
-        if type(target) == int:
+        if isinstance(target, int):
             self.active_preset = target
         else:
             self.active_preset = int(target.get_string()) if target.get_string() != 'eraser' else 0
@@ -792,9 +809,9 @@ class Scribbler(builder.Builder):
 
         # Presenter-side setup
         self.scribble_color_selector.set_rgba(self.scribble_color)
-        self.scribble_width_selector.set_value(self.scribble_width)
+        self.scribble_width_selector.set_value(math.log10(self.scribble_width) if self.scribble_width < 10
+                                               else 1 + (self.scribble_width - 10) / 90)
         self.scribble_color_selector.set_sensitive(target != 'eraser')
-        self.scribble_width_selector.set_sensitive(target != 'eraser')
 
         # Re-draw the eraser
         self.scribble_p_da.queue_draw()
@@ -830,7 +847,7 @@ class Scribbler(builder.Builder):
         """
         button_number = int(widget.get_name().split('_')[-1])
         color, width = self.color_width[button_number]
-        icon, mask = self.marker_surfaces[int((width - 1) / 30)]
+        icon, mask = self.marker_surfaces[int(width >= 2) + int(width >= 40)]
 
         ww, wh = widget.get_allocated_width(), widget.get_allocated_height()
         scale = wh / icon.get_height()
